//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.Graph;

import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.*;

/**
 *	Graphic element
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public abstract class Element
{
	/**
	 *	State of the Element
	 */
	private boolean selected;
	
	
	/**
	 *	Title of the Element (text displayed)
	 */
	private String title;
		
	/**
	 *	Colors
	 */
	private Color[] colors;
		
	
	/**
	 *	Padding
	 */
	private Dimension2D padding;
	
	/**
	 *	The graph
	 */
	private GraphPanel graph;
	
	//--
	
	/**
	 *	Init the Element
	 *
	 *	@param title		the title of the element (the text displayed)
	 */
	public Element(String title)
	{
		this.title = title;
		
		colors = new Color[]{
			Color.black,				// foreground
			Color.white,			// background
			Color.red,				// foreground when selected
			Color.yellow			// background when selected
		};
		
		padding = (Dimension2D)new Dimension(5,5);
	}
	
	/**
	 *	A copy builder
	 *
	 *	@param e		the element to copy
	 */
	public Element(Element e)
	{
		title = e.title;
		colors = e.colors;
		padding = e.padding;
		graph = e.graph;
	}
	
	//--
	
	/**
	 *	Clone the element
	 *
	 *	@return dolly the clone
	 */
	public abstract Object clone();
	
	/**
	 *	Get the title
	 *
	 *	@return	the title
	 */
	public String getTitle()
	{
		return title;
	}
	
	/**
	 *	Set the title
	 *
	 *	@param title	the new title
	 */
	public void setTitle(String title)
	{
		this.title = title;
	}
	
	/**
	 *	Set the state of the Element
	 *
	 *	@param value		the state
	 */
	public void select(boolean value)
	{
		selected = value;
	}
	
	/**
	 *	Get the state of the Element
	 *
	 *	@return	if the Element is selected
	 */
	public boolean isSelected()
	{
		return selected;
	}
	
	/**
	 *	Set the foreground color
	 *
	 *	@param c		color
	 */
	public void setForegroundColor(Color c)
	{
		if(c != null)
			colors[0] = c;
	}
	
	/**
	 *	Set the background color
	 *
	 *	@param c		color
	 */
	public void setBackgroundColor(Color c)
	{
		if(c != null)
			colors[1] = c;
	}
	
	/**
	 *	Set the foreground color when selected
	 *
	 *	@param c		color
	 */
	public void setSelectedForegroundColor(Color c)
	{
		if(c != null)
			colors[3] = c;
	}
	
	/**
	 *	Set the background color when selected
	 *
	 *	@param c		color
	 */
	public void setSelectedBackgroundColor(Color c)
	{
		if(c != null)
			colors[3] = c;
	}
	
	/**
	 *	Get the foreground color
	 *
	 *	@return		color
	 */
	public Color getForegroundColor()
	{
		if(selected == false)
			return colors[0];
		else
			return colors[2];
	}
	
	/**
	 *	Get the background color
	 *
	 *	@return color
	 */
	public Color getBackgroundColor()
	{
		if(selected == false)
			return colors[1];
		else
			return colors[3];
	}
	
	/**
	 *	Set padding (internal margin)
	 *
	 *	@param padding		the padding
	 */
	public void setPadding(Dimension2D padding)
	{
		this.padding = padding;
	}
	
	/**
	 *	Get the padding
	 *
	 *	@return padding
	 */
	public Dimension2D getPadding()
	{
		return padding;
	}
	
	//--
	
	/**
	 *	Draw the element
	 *
	 *	@param g2d	Graphic context
	 */
	public abstract void draw(Graphics2D g2d);
	
	/**
	 *	Test if the Element contains the point
	 *
	 *	@param p		the point
	 *	@return 		if the Element contains this point
	 */
	public abstract boolean contains(Point2D p);
		
	/**
	 *	Display a dialog box for this node
	 *
	 *	@param graph		the graph
	 */
	public void showDialog(GraphPanel graph)
	{
		this.graph = graph;
	}
		
	/**
	 *	Get the graph
	 */
	public GraphPanel getGraph()
	{
		return graph;
	}
	
	/**
	 * Update the values
	 */
	public void update()
	{
		if(graph !=null)
			graph.redraw();
	}
}