//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.RDF;

import ch.hearc.I04_05s08.GUI.Graph.*;
import ch.hearc.I04_05s08.Ontology.RDF.*;
import ch.hearc.I04_05s08.Ontology.RDFS.RdfsFile;
import com.hp.hpl.jena.rdf.model.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.Iterator;

/**
 *	The graph for a RDF
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfGraph extends GraphPanel
{
	/**
	 *	RDF
	 */
	private RdfFile file;
	
	/**
	 *	the tree
	 */
	private JTree tree;
	
	/**
	 *	Source display
	 */
	private JTextPane tp;
	
	/**
	 *	Colors
	 */
	private Color[][] colors;
	
	/**
	 *	Build
	 *
	 *	@param file			RDF
	 *	@param textpane		text area for displaying sources
	 */
	public RdfGraph(RdfFile file, JTextPane textpane)
	{
		super();
		
		if(file != null)
			this.file = file;
		else
			this.file = new RdfFile("untitled Schema");
	
		if(file.getRdfs() != null)
			colors = new Color[file.getRdfs().length][2];
		
		tp = textpane;
		update();
	}
	
	/**
	 *	Build
	 *
	 *	@param file		RDF
	 */
	public RdfGraph(RdfFile file)
	{
		this(file, null);
	}
	
	/**
	 *	Build
	 *
	 *	@param name 		filename of the RDF
	 */
	public RdfGraph(String name)
	{
		this(null, null);
	}
	
	/**
	 *	Add a RDF node
	 *
	 *	@param n		Node
	 */
	public boolean add(RdfNode n)
	{
		boolean val = (file.add(n.getInstanceOfClass())
			? super.add((Node)n)
			: false);
		
		update();
		return val;
	}
	
	/**
	 *	Add RDF nodes
	 *
	 *	@param n		Nodes
	 */
	public boolean add(RdfNode[] n)
	{
		boolean val = super.add(n);
		
		update();
		return val;
	}
	
	/**
	 *	Add a RDF arrow
	 *
	 *	@param a		Arrow
	 */
	public boolean add(RdfArrow a)
	{		
		boolean val = (file.add(a.getInstanceOfProperty())
			? super.add((Arrow)a)
			: false);
		
		update();
		return val;
	}
	
	/**
	 *	Add RDFS arrows
	 *
	 *	@param a		Arrow
	 */
	public boolean add(RdfArrow[] a)
	{
		boolean val = super.add(a);
		
		update();
		return val;
	}
	
	/**
	 *	Add an Element to the graph
	 *
	 *	@param e		the element
	 */
	public boolean add(Element e)
	{
		boolean val = true;
		
		if(e instanceof RdfArrow &&
			getSelectedElement() instanceof RdfNode &&
			getOldSelectedElement() instanceof RdfNode)
		{
			if(((RdfArrow)e).setNodes(
					(RdfNode)getOldSelectedElement(),
				(RdfNode)getSelectedElement())
			)
			{
				RdfArrow arrow = (RdfArrow)e.clone();
				arrow.showDialog((GraphPanel)this);
				val = add(arrow);
			}
		}
		else if(e instanceof RdfNode)
		{
			RdfNode node = (RdfNode)e.clone();
			node.showDialog((GraphPanel)this);
			val = add(node);
		}
		
		if(tree != null)
			tree.setSelectionRow(0);
		
		update();
		return val;
	}
	
	/**
	 *	Remove an arrow
	 *
	 *	@param a		arrow
	 *	@return if the arrow sucessfully removed
	 */
	public boolean del(RdfArrow a)
	{
		a.unconnect();
		update();
		return super.del(a);
	}
	
	/**
	 *	Delete a node from the schema
	 *
	 *	@param n		the node
	 */
	public boolean del(RdfNode n)
	{
		boolean val = super.del(n) && file.del(n.getInstanceOfClass());
		update();
		return val;
	}
	
	/**
	 *	Delete nodes from the schema
	 *
	 *	@param n		some nodes
	 */
	public boolean del(RdfNode[] n)
	{
		boolean val=true;
		
		for(int i=0; i<n.length; i++)
			val &= del(n[i]);
		
		return val;
	}
	
	/**
	 *	Remove some arrows
	 *
	 *	@param a		arrows
	 *	@return if the arrows sucessfully removed
	 */
	public boolean del(RdfArrow[] a)
	{
		return super.del(a);
	}
	
	/**
	 *	Add an Element to the graph
	 *
	 *	@param e	the element
	 */
	public boolean del(Element e)
	{
		boolean val = true;
		
		if(e instanceof RdfArrow)
		{
			val = del((RdfArrow)e);
		}
		else if(e instanceof RdfNode)
		{
			val = del((RdfNode)e);
		}
		
		return val;
	}
	
	/**
	 *	Update the graph
	 */
	public void update()
	{
		if(tp != null)
			tp.setText(RdfManager.rdfToString(file));
		
		redraw();
	}
	
	/**
	 *	Redraw
	 */
	public void redraw()
	{
		super.redraw();
	}
	
	/**
	 *	Get the node
	 *
	 *	@param ioc		the instance of class
	 *	@return the node
	 */
	public RdfNode getNode(InstanceOfClass ioc)
	{
		RdfNode out = null;
		RdfNode work;
		
		for(Iterator i = getNodes().iterator(); i.hasNext();)
		{
			work = (RdfNode)i.next();
			if(work instanceof RdfResource && work.getInstanceOfClass().getValue().equals(ioc.getValue()))
			{
				out = work;
				break;
			}
		}
		
		// Add the good color for the good Schema
		if(out != null)
		{
			RdfsFile[] rdfs = file.getRdfs();
			
			if(rdfs != null)
				for(int i=0; i<rdfs.length; i++)
				{
					if(rdfs[i].getClass(ioc.getName()) != null && colors[i] != null)
					{
						out.setBackgroundColor(colors[i][0]);
						out.setForegroundColor(colors[i][1]);
					}
				}
		}
		
		return out;
	}
	
	/**
	 *	Build an arrow for the specified property (usefull to have to right color)
	 *
	 *	@param iop		the instance of property
	 *	@return the arrow
	 */
	public RdfArrow getArrow(InstanceOfProperty iop)
	{
		RdfsFile[] rdfs = file.getRdfs();
		
		RdfArrow out = new RdfArrow(iop);
		
		if(rdfs != null)
			for(int i=0; i<rdfs.length; i++)
			{
				if(rdfs[i].getProperty(iop.getName()) != null && colors[i] != null)
				{
					out.setBackgroundColor(colors[i][0]);
					out.setForegroundColor(colors[i][1]);
				}
			}
		
		return out;
	}
	
	/**
	 *	Set the tree
	 *
	 *	@param tree		the tree
	 */
	public void setTree(JTree tree)
	{
		this.tree = tree;
	}
	
	/**
	 *	Add a color for (RDF Schema)
	 *	The need to be in the same order of the RDF Schema into the RDF file.
	 *
	 *	@param color		color
	 */
	public void addColor(Color[] color)
	{
		if(color.length == 2)
		{
			for(int i=0; i<colors.length; i++)
			{
				if(colors[i][0] == null)
				{
					colors[i][0] = color[0];
					colors[i][1] = color[1];
					break;
				}
			}
		}
	}
}
