//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDF;

import java.util.LinkedList;
import java.util.Iterator;
import ch.hearc.I04_05s08.Ontology.RDFS.*;

/**
 *	A RDF Resource that represent a property of a Class
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class InstanceOfProperty extends Instance
{
	/**
	 * The element that this property is applied
	 */
	private InstanceOfClass propertyOf;
	
	/**
	 * Value of the property
	 */
	private InstanceOfClass value;
	
	/**
	 *	URI of the RDF#type
	 */
	public static final String type = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 * Construct an instance of the prop that belong to propOf
	 *
	 * @param prop		Property instancied
	 */
	public InstanceOfProperty(RdfsProperty prop)
	{
		super(prop, null);
	}
	
	/**
	 * Construct an instance of the prop that belong to an unknowned Property
	 *
	 * @param prop		Property instancied
	 */
	public InstanceOfProperty(String prop)
	{
		super(prop);
	}
	
	/**
	 * Make this instance of copy of another
	 *
	 * @param iop			the instance copied
	 */
	public InstanceOfProperty(InstanceOfProperty iop)
	{
		super(iop);
		this.propertyOf = iop.propertyOf;
		this.value = iop.value;
	}
	
	// Public /////////////////////////////////////////////////////////////////
	
	/**
	 *	Build a clone
	 *
	 *	@return a clone
	 */
	public Object clone()
	{
		return (Object)new InstanceOfProperty(this);
	}
	
	/**
	 *	String representation
	 *
	 *	@return string representation
	 */
	public String toString()
	{
		String str = "";
		
		if(isInstance())
			str += "(rdfs) ";
		
		str += getName();
		
		if(propertyOf != null)
		{
			str += "\n propOf : " + propertyOf;
			str += "\n value : " + value;
		}
		
		return str;
	}
	
	/**
	 * Set the value of the property
	 *
	 * @param value	the value that would be applied to Property
	 * @return 		if the value could be applied
	 */
	public boolean setProperty(InstanceOfClass value)
	{
		if(checkRange(value))
		{
			this.value = value;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Get the value of the property
	 *
	 * @return 		the property's value
	 */
	public InstanceOfClass getProperty()
	{
		return value;
	}
	
	/**
	 * Set the value of the property
	 *
	 * @param propertyOf	the value that would be applied to Property
	 * @return 				if the value could be applied
	 */
	public boolean setPropertyOf(InstanceOfClass propertyOf)
	{
		if(checkDomain(propertyOf))
		{
			this.propertyOf = propertyOf;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Get the instance of the class is this property applied
	 *
	 * @return 		the class's instance of this property
	 */
	public InstanceOfClass getPropertyOf()
	{
		return propertyOf;
	}
	
	/**
	 * Check the conjunction of domains and the class
	 *
	 * @param c		the class to be checked
	 * @return 		if the class respect the conjuction of domains' property
	 */
	private boolean checkDomain(InstanceOfClass c)
	{
		if(isInstance())
		{
			return c.isA(((RdfsProperty)getResource()).getDomain());
		}
		else
		{
			return true;
		}
	}
	
	/**
	 * Check the conjunction of ranges and the class
	 *
	 * @param c	the class to be checked
	 * @return if the class respect the conjuction of ranges' property
	 */
	private boolean checkRange(InstanceOfClass c)
	{
		if(isInstance())
		{
			return c.isA(((RdfsProperty)getResource()).getRange());
		}
		else
		{
			return true;
		}
	}
}
