//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDFS;

import java.util.LinkedList;
import java.util.Iterator;
import ch.hearc.I04_05s08.Ontology.OntFile;

/**
 *	A representation of a rdfs:Class
 *
 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_class">w3c</a>
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfsFile extends OntFile
{
	/**
	 *	Classes composing the RDF Schema
	 */
	private LinkedList classes;
	
	/**
	 *	Properties composing the RDF Schema
	 */
	private LinkedList properties;
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 *	Init the RDF Schema file
	 * 
	 *	@param filename		the file's path
	 */
	public RdfsFile(String filename)
	{
		super(filename);
		
		this.classes = new LinkedList();
		this.properties = new LinkedList();
	}
	
	/**
	 *	Init the untitled RDF Schema file
	 */
	public RdfsFile()
	{
		this("");
	}

	// Public /////////////////////////////////////////////////////////////////
	
	/**
	 *	String representation of the rdfs file
	 *
	 *	@return			each classes and properties
	 */
	public String toString()
	{
		String str=super.toString();
		
		str += "\nClass :\n";
		for(Iterator i = classes.iterator(); i.hasNext();)
		{
			str += i.next().toString() + "\n";
		}
		
		str += "\nProperty :\n";
		for(Iterator i = properties.iterator(); i.hasNext();)
		{
			str += i.next().toString() + "\n";
		}
		
		return str;
	}
	
	/**
	 *	Delete a resource from the file
	 *
	 *	@param r	resource
	 *	@return if the resource isn't already out
	 */
	public boolean del(RdfsResource r)
	{
		if(r instanceof RdfsClass)
		{
			return del((RdfsClass)r);
		}
		else if(r instanceof RdfsProperty)
		{
			return del((RdfsProperty)r);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Add a resource to the file
	 *
	 *	@param r	resource
	 *	@return if the resource isn't already in
	 */
	public boolean add(RdfsResource r)
	{
		if(r instanceof RdfsClass)
		{
			return add((RdfsClass)r);
		}
		else if(r instanceof RdfsProperty)
		{
			return add((RdfsProperty)r);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Add a new Class to the RDF Schema
	 *
	 *	@param c		the class which would be add
	 *	@return if the Class wasn't already listed
	 */
	public boolean add(RdfsClass c)
	{
		if(!classes.contains(c) && getClass(c.getName()) == null)
		{
			return classes.add(c);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Remove a Class from the RDF Schema
	 *
	 *	@param c		the class which would be deleted
	 *	@return if the Class was successfuly removed
	 */
	public boolean del(RdfsClass c)
	{
		return classes.remove(c);
	}
	
	/**
	 *	Get all classes members of the RDF Schema
	 *
	 *	@return the classes
	 */
	public RdfsClass[] getClasses()
	{
		RdfsClass[] allclasses;
		Iterator i;
		
		allclasses = new RdfsClass[classes.size()];
		
		i = classes.iterator();
		int j=0;
		while(i.hasNext())
		{
			allclasses[j] = (RdfsClass)i.next();
			j++;
		}
		
		return allclasses;
	}
	
	/**
	 *	Get a class from is name
	 *
	 *	@param name		the name of the desired class
	 *	@return the class
	 */
	public RdfsClass getClass(String name)
	{
		RdfsClass c = null;
		RdfsClass d;
		
		for(Iterator i = classes.iterator(); i.hasNext();)
		{
			d= (RdfsClass)i.next();
			if(d.getName().equals(name))
			{
				c = d;
				break;
			}
		}
		return c;
	}
	
	/**
	 *	Return if the RdfsClass is already in the file
	 *
	 *	@return if the rdfs is already in
	 */
	public boolean containsClass(RdfsClass c)
	{
		return (classes.contains(c));
	}
	
	/**
	 *	Add a new Property to the RDF Schema
	 *
	 *	@param prop		the property which would be add
	 *	@return if the Property was successfuly added
	 */
	public boolean add(RdfsProperty prop)
	{
		if(!properties.contains(prop) && getProperty(prop.getName()) == null)
		{
			return properties.add(prop);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Delete the Property
	 *
	 *	@param prop		the property which would be deleted
	 *	@return if the Property was successfuly deleted
	 */
	public boolean del(RdfsProperty prop)
	{
		return properties.remove(prop);
	}
	
	/**
	 *	Get all properties members of the RDF Schema
	 *
	 *	@return the properties
	 */
	public RdfsProperty[] getProperties()
	{
		RdfsProperty[] allproperties;
		Iterator i;
		
		allproperties = new RdfsProperty[properties.size()];
		
		i = properties.iterator();
		int j=0;
		while(i.hasNext())
		{
			allproperties[j] = (RdfsProperty)i.next();
			j++;
		}
		
		return allproperties;
	}
	
	/**
	 *	Get a property from is name
	 *
	 *	@param name		the name of the desired property
	 *	@return the property
	 */
	public RdfsProperty getProperty(String name)
	{
		RdfsProperty p = null, d;
		for(Iterator i = properties.iterator(); i.hasNext();)
		{
			d= (RdfsProperty)i.next();
			if(d.getName().equals(name))
			{
				p = d;
				break;
			}
		}
		return p;
	}
}
