//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDFS;

import java.util.LinkedList;
import java.util.Iterator;

/**
 *	Class that represent a Resource.
 *	Every Classes of any RDFS is a sub classe of it.
 *	
 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_resource">w3c</a>
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public abstract class RdfsResource
{
	/**
	 *	Name of the resource (rdf:about)
	 */
	private String name;
	
	/**
	 *	A human-readable version of resource's name
	 *	
	 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_label">w3c</a>
	 */
	private String label;
	
	/**
	 *	Description of resource's name
	 *	
	 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_comment">w3c</a>
	 */
	private String comment;
	
	/**
	 *	Minimal constructor with the name only
	 *	
	 *	@param name		the resource's name
	 */
	public RdfsResource(String name)
	{
		init(name, null,null);
	}
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 *	Minimal constructor recommanded
	 *	
	 *	@param name		the resource's name
	 *	@param label	human readable name
	 */
	public RdfsResource(String name, String label)
	{
		init(name, label, null);
	}
	
	/**
	 *	Full constructor recommanded
	 *	
	 *	@param name		the resource's name
	 *	@param label	human readable name
	 *	@param comment	description
	 */
	public RdfsResource(String name, String label, String comment)
	{
		init(name, label,comment);
	}
	
	/**
	 * Build a new resource from a another
	 *
	 * @param r		resource source
	 */
	public RdfsResource(RdfsResource r)
	{
		init(r.name, r.label, r.comment);
	}
	
	// Public /////////////////////////////////////////////////////////////////

	/**
	 *	String representation of the Resource
	 *
	 *	@return		name, label and comment
	 */
	public String toString()
	{
		String str = new String(name);
		
		if(label != null)
			str += "\n\tlabel : " + label;
		
		if(comment != null)
			str += "\n\tcomment : " + comment;
		
		return str;
	}
	
	/**
	 * Clone the Resource
	 *
	 * @return a copy
	 */
	public abstract Object clone();
	
	/**
	 * Get the name of the resource
	 *
	 * @return name of the resource
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 *	Set the name of the resource
	 *
	 *	@param name		name of the resource
	 */
	public void setName(String name)
	{
		if(!name.equals(""))
			this.name = name;
	}
	
	/**
	 *	Get the short name
	 *
	 *	@return	the short name of the Resource
	 */
	public String getShortName()
	{
		String[] res;
		res = name.split("#");
		if(res.length == 1)
		{
			res = name.split("/");
		}
		
		return (res.length >= 1) ? res[res.length-1] : name;
	}
	
	/**
	 * Set the label of the resource
	 *
	 * @param label	human readable name
	 */
	public void setLabel(String label)
	{
		if(!label.equals(""))
			this.label = label;
		else
			this.label = null;
	}
	
	/**
	 * Get the label of the resource
	 *
	 * @return human readable name
	 */
	public String getLabel()
	{
		return label;
	}
	
	/**
	 * Set the comment of the resource
	 *
	 * @param comment	description
	 */
	public void setComment(String comment)
	{
		if(!comment.equals(""))
			this.comment = comment;
		else
			this.comment = null;
	}	
	
	/**
	 * Get the comment of the resource
	 *
	 * @return description
	 */
	public String getComment()
	{
		return comment;
	}
	
	// Private ////////////////////////////////////////////////////////////////
	
	/**
	 *	Init the Resource
	 *	
	 *	@param	label		
	 *	@param	comment
	 */
	private void init(String name, String label, String comment)
	{
		this.name = name;
		this.label = label;
		this.comment = comment;
	}
	
	// Public static //////////////////////////////////////////////////////////
	
	/**
	 *	Copy a LinkedList into a new List
	 *
	 *	@param	source	LinkedList of source
	 *	@return	copy of the LinkedList source
	 */
	public static LinkedList copyList(LinkedList source)
	{
		LinkedList dest;
		Iterator i;
		
		dest = new LinkedList();
		
		i = source.iterator();
		while(i.hasNext())
		{
			// No cloning action because it's a graph any deep copy
			// will cut it off.
			dest.add(i.next());
		}
		
		return dest;
	}
	
	/**
	 *	If the resource is the same as this one
	 *
	 *	@param resource		a resource
	 *	@return if it's the same resource
	 */
	public boolean equals(RdfsResource resource)
	{
		return name.equals(resource.name);
	}
}
